#include <pspkernel.h>

#include <systemctrl.h>
#include <systemctrl_se.h>

#include <string.h>

PSP_MODULE_INFO("SysconfPatcher", 0x1007, 1, 0);

#define MAKE_CALL(a, f) _sw(0x0C000000 | (((u32)(f) >> 2)  & 0x03ffffff), a)
#define PSP_FIRMWARE(f) ((((f >> 8) & 0xF) << 24) | (((f >> 4) & 0xF) << 16) | ((f & 0xF) << 8) | 0x10)

STMOD_HANDLER previous = NULL;

char *ver_info = NULL;
char *mac_info = NULL;

typedef struct
{
	u32 devkit;
	u32 sysconf_ver;
	u32 sysconf_mac;
} PatchesVersion;

PatchesVersion sysconfPatches[] =
{
	//5.00
	{ PSP_FIRMWARE(0x500), 0x15EE0, 0x15C10 },
	
	//5.02, sysconf, didn't change =/
	{ PSP_FIRMWARE(0x502), 0x15EE0, 0x15C10 },
	
	//5.03, sysconf didn't change... again
	{ PSP_FIRMWARE(0x503), 0x15EE0, 0x15C10 },
	
	//5.05 sysconf didn't change... again... again
	{ PSP_FIRMWARE(0x505), 0x15EE0, 0x15C10 },
};

#define PATCHES_N (sizeof(sysconfPatches) / sizeof(PatchesVersion))

int OnModuleStart(SceModule2 *mod)
{
	int i;
	u32 ver = sceKernelDevkitVersion();

	if (strcmp(mod->modname, "sysconf_plugin_module") == 0)
	{
		for (i = 0; i < PATCHES_N; i++)
		{
			if (sysconfPatches[i].devkit == ver)
			{			
				if (ver_info)
				{
					_sw(0x3C020000 | ((int)ver_info >> 16), mod->text_addr + sysconfPatches[i].sysconf_ver);
					_sw(0x34420000 | ((int)ver_info & 0xFFFF), mod->text_addr + sysconfPatches[i].sysconf_ver + 4);
				}
				
				if (mac_info)
				{
					_sw(0x3C060000 | ((int)mac_info >> 16), mod->text_addr + sysconfPatches[i].sysconf_mac);
					_sw(0x24C60000 | ((int)mac_info & 0xFFFF), mod->text_addr + sysconfPatches[i].sysconf_mac + 4);
				}
				
				sceKernelDcacheWritebackAll();
				sceKernelIcacheClearAll();
			}
		}
	}
	
	if (!previous)
		return 0;
	
	return previous(mod);
}

int getSpoof(char *file, int mode)
{
	int i;
	char *global;
	u16 isunicode = 0;
	
	SceIoStat stat;
	memset(&stat, 0, sizeof(SceIoStat));
	
	if (sceIoGetstat(file, &stat) < 0)
		return -1;
	
	SceUID fd = sceIoOpen(file, PSP_O_RDONLY, 0777);
	
	if (fd < 0)
		return -1;
	
	sceIoRead(fd, &isunicode, sizeof(u16));
	
	if (isunicode != 0xFEFF)
	{
		isunicode = 0;
		stat.st_size = (stat.st_size * 2) + 2;
		sceIoLseek32(fd, 0, PSP_SEEK_SET);
	}

	SceUID block_id = sceKernelAllocPartitionMemory(2, "", PSP_SMEM_Low, stat.st_size, NULL);
	
	if (block_id < 0)
		return -1;
	
	global = sceKernelGetBlockHeadAddr(block_id);
	memset(global, 0, stat.st_size);	
	
	if (isunicode)
	{
		sceIoRead(fd, global, stat.st_size - 2);
	}
	
	else
	{
		for (i = 0; i < (stat.st_size / 2); i++)
		{
			sceIoRead(fd, global + (i * 2), 1);
		}
	}
	
	if (!mode)
		ver_info = global; //lazy
	
	else
		mac_info = global;
	
	sceIoClose(fd);
	return 0;
}

int module_start(SceSize args, void *argp)
{	
	
	int ret = getSpoof("ms0:/seplugins/spoofversion.txt", 0);
	ret += getSpoof("ms0:/seplugins/spoofmac.txt", 1);
	
	if (ret < -1)
		return ret;

	previous = sctrlHENSetStartModuleHandler(OnModuleStart);
	return 0;
}
